@@ -116,7 +116,7 @@ impl Registry {
                     r += &format!(">\n{}    </property>\n",  x.annotations.introspect("      "));
                 }
             }
-            desc.annotations.introspect("    ");
+            r += &desc.annotations.introspect("    ");
             r += "  </interface>\n";
         };
         r
@@ -311,7 +311,7 @@ impl<T: Send, A: Send + arg::RefArg + arg::Arg + arg::Append> PropBuilder<'_, T,
         })
     }
 
-    pub (crate) fn get_custom<CB>(mut self, mut cb: CB) -> Self
+    pub (crate) fn get_custom<CB>(self, mut cb: CB) -> Self
     where CB: FnMut(PropContext, &mut Crossroads) -> Option<PropContext> + Send + 'static {
         self.desc.get_cb = Some(Dbg(Box::new(move |ctx, cr| {
             cb(ctx, cr)
@@ -410,7 +410,7 @@ impl<T: Send, A: arg::RefArg + Send + for<'x> arg::Get<'x> + arg::Arg + arg::App
         })
     }
 
-    pub fn set_custom<CB>(mut self, mut cb: CB) -> Self
+    pub fn set_custom<CB>(self, mut cb: CB) -> Self
     where CB: FnMut(PropContext, &mut Crossroads, A) -> Option<PropContext> + Send + 'static {
         self.desc.set_cb = Some(Dbg(Box::new(move |mut ctx, cr| {
             match ctx.check(|ctx| {
@@ -610,11 +610,11 @@ impl<T: Send + 'static> IfaceBuilder<T> {
         }
     }
 
-    pub fn annotate<N: Into<String>, V: Into<String>>(mut self, name: N, value: V) -> Self {
+    pub fn annotate<N: Into<String>, V: Into<String>>(&mut self, name: N, value: V) -> &mut Self {
         self.0.annotations.insert(name, value);
         self
     }
-    pub fn deprecated(self) -> Self { self.annotate(DEPRECATED, "true") }
+    pub fn deprecated(&mut self) -> &mut Self { self.annotate(DEPRECATED, "true") }
 
     pub (crate) fn build<F>(name: Option<strings::Interface<'static>>, f: F) -> IfaceDesc
     where F: FnOnce(&mut IfaceBuilder<T>) {
