@@ -9,7 +9,7 @@ use std::marker::PhantomData;
 use crate::ifacedesc::EMITS_CHANGED;
 
 fn introspect(cr: &Crossroads, path: &dbus::Path<'static>) -> String {
-    let mut children = cr.get_children(path);
+    let mut children = cr.get_children(path, true);
     let mut childstr = String::new();
     children.sort_unstable();
     for c in children {
@@ -51,7 +51,6 @@ pub (crate) fn make_emits_message<V: dbus::arg::Arg + dbus::arg::Append>(prop_na
         .append3(s, dbus::arg::Dict::new(d), i))
 }
 
-
 #[derive(Debug)]
 /// PropContext is a struct that provides helpful information inside a get/set property handler.
 ///
@@ -112,6 +111,13 @@ impl PropContext {
     }
 
     pub (crate) fn set_send_on_drop(&mut self, value: Arc<dyn Sender + Send + Sync>) {
+        if let Some(get_all) = &self.get_all {
+            let mut pactx = get_all.lock().unwrap();
+            if let Some(ref mut propctx) = pactx.propctx {
+                propctx.context.as_mut().map(|ctx| ctx.set_send_on_drop(value));
+                return;
+            }
+        }
         self.context.as_mut().map(|ctx| ctx.set_send_on_drop(value));
     }
 
@@ -140,6 +146,7 @@ impl PropContext {
         };
         let octx = cb(self, cr);
         cr.registry().give_prop(token, &name, cb, is_set);
+        // dbg!(&name, octx.is_some());
         octx
     }
 
@@ -184,6 +191,13 @@ impl PropContext {
             },
         }
     }
+
+    /// The underlying Context, if any.
+    ///
+    /// This gives you access to additional methods (e.g., push_msg and make_signal).
+    pub fn context_mut(&mut self) -> Option<&mut Context> {
+        self.context.as_mut()
+    }
 }
 
 #[derive(Debug)]
@@ -221,7 +235,19 @@ fn get(mut ctx: Context, cr: &mut Crossroads, (interface_name, property_name): (
     propctx.call_prop(cr, false).map(|propctx| { propctx.context.unwrap() })
 }
 
+fn getall_all(ctx: Context, cr: &mut Crossroads) -> Option<Context> {
+    get_all_for_path(&ctx.path().clone(), cr, Some(ctx), move |ictx, octx| {
+        let props: HashMap<_, _> = ictx.ifaces.values().flatten().collect();
+        octx.as_mut().unwrap().do_reply(|msg| {
+            msg.append_all((props,));
+        });
+    })
+}
+
 fn getall(mut ctx: Context, cr: &mut Crossroads, (interface_name,): (String,)) -> Option<Context> {
+    if interface_name == "" {
+        return getall_all(ctx, cr);
+    }
     let mut propctx = match ctx.check(|ctx| { PropContext::new(cr, ctx.path().clone(), interface_name, "".into())}) {
         Ok(p) => p,
         Err(_) => return Some(ctx),
@@ -267,17 +293,24 @@ pub fn properties(cr: &mut Crossroads) -> IfaceToken<()> {
 struct IfaceContext {
     remaining: usize,
     ifaces: IfacePropMap,
-    donefn: Option<Dbg<Box<dyn FnOnce(&mut IfaceContext) + Send + 'static>>>,
+    donefn: Option<Dbg<Box<dyn FnOnce(&mut IfaceContext, &mut Option<Context>) + Send + 'static>>>,
 }
 
 type IfacePropMap = HashMap<String, PropMap>;
 type PathPropMap = HashMap<dbus::Path<'static>, IfacePropMap>;
 
-fn get_all_for_path<F: FnOnce(&mut IfaceContext) + Send + 'static>(path: &dbus::Path<'static>, cr: &mut Crossroads, f: F) {
+fn get_all_for_path<F>(path: &dbus::Path<'static>, cr: &mut Crossroads, octx: Option<Context>, f: F) -> Option<Context>
+where F: FnOnce(&mut IfaceContext, &mut Option<Context>) + Send + 'static {
+    let (_reg, ifaces) = cr.registry_and_ifaces(&path);
+    let all: Vec<usize> = ifaces.into_iter().map(|token| *token).collect();
+    for_each_interface_with_properties(path, all, cr, octx, f)
+}
+
+fn for_each_interface_with_properties<F: FnOnce(&mut IfaceContext, &mut Option<Context>) + Send + 'static, I: IntoIterator<Item = usize>>
+(path: &dbus::Path<'static>, ifaces: I, cr: &mut Crossroads, mut octx: Option<Context>, f: F) -> Option<Context> {
     let ictx: Arc<Mutex<IfaceContext>> = Default::default();
-    let (reg, ifaces) = cr.registry_and_ifaces(&path);
-    let all: Vec<_> = ifaces.into_iter().filter_map(|&token| {
-        if !reg.has_props(token) { return None };
+    let (reg, _all_ifaces) = cr.registry_and_ifaces(&path);
+    let all: Vec<_> = ifaces.into_iter().filter_map(|token| {
         let iface_name = reg.get_intf_name(token)?;
         Some(PropContext {
             context: None,
@@ -291,35 +324,50 @@ fn get_all_for_path<F: FnOnce(&mut IfaceContext) + Send + 'static>(path: &dbus::
     }).collect();
 
     if all.len() == 0 {
-        f(&mut *ictx.lock().unwrap());
-        return;
+        f(&mut *ictx.lock().unwrap(), &mut octx);
+        return octx;
     }
 
     let mut ic = ictx.lock().unwrap();
     ic.remaining = all.len();
     ic.donefn = Some(Dbg(Box::new(f)));
     drop(ic);
-    for pctx in all.into_iter() {
+    let amoctx = Arc::new(Mutex::new(octx));
+    for mut pctx in all.into_iter() {
         let iclone = ictx.clone();
+        let amoclone = amoctx.clone();
+        pctx.context = amoctx.lock().unwrap().take();
         pctx.call_all_props(cr, move |pactx| {
             let mut ic = iclone.lock().unwrap();
+            let mut amoctx = amoclone.lock().unwrap();
+            if amoctx.is_none() {
+                *amoctx = pactx.propctx.as_mut().unwrap().context.take();
+            }
             let answers = std::mem::replace(&mut pactx.answers, HashMap::new());
-            // dbg!(&pactx);
+            //dbg!(amoctx.is_some(), &answers);
             ic.ifaces.insert(pactx.propctx.as_ref().unwrap().interface.to_string(), answers);
             ic.remaining -= 1;
             if ic.remaining == 0 {
                 let donefn = ic.donefn.take().unwrap().0;
-                (donefn)(&mut *ic)
+                (donefn)(&mut *ic, &mut amoctx)
+            }
+        }).map(|mut pctx| {
+            let mut amoctx = amoctx.lock().unwrap();
+            if amoctx.is_none() {
+                *amoctx = pctx.context.take();
             }
         });
     }
+    let mut amoctx = amoctx.lock().unwrap();
+    amoctx.take()
 }
+
 //
 fn get_managed_objects(mut ctx: Context, cr: &mut Crossroads, _: ()) -> Option<Context> {
     // HashMap<dbus::Path<'static>, IfacePropMap>
     let parent = ctx.path();
     let children: Vec<dbus::Path<'static>> =
-        cr.get_children(ctx.path()).into_iter().map(|child_path| {
+        cr.get_children(ctx.path(), false).into_iter().map(|child_path| {
             let mut x = String::from(&**parent);
             if !x.ends_with('/') {
                 x.push_str("/");
@@ -340,34 +388,37 @@ fn get_managed_objects(mut ctx: Context, cr: &mut Crossroads, _: ()) -> Option<C
     struct Temp {
         remaining: usize,
         temp_map: PathPropMap,
-        ctx: Option<Context>,
+        octx: Option<Context>,
     }
     let r = Arc::new(Mutex::new(Temp {
         remaining: children.len(),
         temp_map: HashMap::new(),
-        ctx: Some(ctx),
+        octx: Some(ctx),
     }));
-    let returned_ctx: Arc<Mutex<Option<Context>>> = Default::default();
     for subpath in children {
         let rclone = r.clone();
-        let rctx = returned_ctx.clone();
         let subpath_clone = subpath.clone();
-        get_all_for_path(&subpath, cr, move |ictx| {
+        let octx = r.lock().unwrap().octx.take();
+        get_all_for_path(&subpath, cr, octx, move |ictx, octx| {
             let mut rr = rclone.lock().unwrap();
+            if rr.octx.is_none() { rr.octx = octx.take(); }
             let ifaces = std::mem::replace(&mut ictx.ifaces, HashMap::new());
             rr.temp_map.insert(subpath_clone, ifaces);
             rr.remaining -= 1;
             // dbg!(&rr);
             if rr.remaining > 0 { return; }
-            let mut ctx = rr.ctx.take().unwrap();
-            ctx.do_reply(|msg| {
+            let mut octx = rr.octx.take().unwrap();
+            octx.do_reply(|msg| {
                 msg.append_all((&rr.temp_map,));
             });
-            *rctx.lock().unwrap() = Some(ctx);
+            rr.octx = Some(octx);
+        }).map(|octx| {
+            let mut rr = r.lock().unwrap();
+            if rr.octx.is_none() { rr.octx = Some(octx); }
         });
     }
-    let mut lock = returned_ctx.lock().unwrap();
-    lock.take()
+    let mut rr = r.lock().unwrap();
+    rr.octx.take()
 }
 
 pub fn object_manager(cr: &mut Crossroads) -> IfaceToken<()> {
@@ -394,7 +445,7 @@ pub fn object_manager_path_added(sender: Arc<dyn Sender + Send + Sync>, name: &d
     object_manager_parents(name, cr, |parent, cr| {
         let n = name.clone();
         let s = sender.clone();
-        get_all_for_path(&name, cr, move |ictx| {
+        get_all_for_path(&name, cr, None, move |ictx, _| {
             let x = dbus::blocking::stdintf::org_freedesktop_dbus::ObjectManagerInterfacesAdded {
                 object: n,
                 interfaces: std::mem::replace(&mut ictx.ifaces, HashMap::new()),
@@ -418,3 +469,32 @@ pub fn object_manager_path_removed(sender: Arc<dyn Sender + Send + Sync>, name:
         let _ = sender.send(dbus::message::SignalArgs::to_emit_message(&x, &parent));
     });
 }
+
+pub fn object_manager_interface_added(sender: Arc<dyn Sender + Send + Sync>, name: &dbus::Path<'static>, itoken: usize, cr: &mut Crossroads) {
+    object_manager_parents(name, cr, |parent, cr| {
+        let n = name.clone();
+        let s = sender.clone();
+
+        for_each_interface_with_properties(&name, vec![itoken], cr, None, move |ictx, _| {
+            let x = dbus::blocking::stdintf::org_freedesktop_dbus::ObjectManagerInterfacesAdded {
+                object: n,
+                interfaces: std::mem::replace(&mut ictx.ifaces, HashMap::new()),
+            };
+            let _ = s.send(dbus::message::SignalArgs::to_emit_message(&x, &parent));
+        });
+    });
+}
+
+pub fn object_manager_interface_removed(sender: Arc<dyn Sender + Send + Sync>, name: &dbus::Path<'static>, itoken: usize, cr: &mut Crossroads) {
+    object_manager_parents(name, cr, |parent, cr| {
+        let (reg, _ifaces) = cr.registry_and_ifaces(&name);
+
+        if let Some(iface) = reg.get_intf_name(itoken) {
+            let x = dbus::blocking::stdintf::org_freedesktop_dbus::ObjectManagerInterfacesRemoved {
+                object: name.clone(),
+                interfaces: vec![iface.to_string()],
+            };
+            let _ = sender.send(dbus::message::SignalArgs::to_emit_message(&x, &parent));
+        }
+    });
+}
